{%- import 'syntax.sql.jinja' as syntax -%}
SELECT table_name
FROM information_schema.tables
WHERE
    table_schema = '{{ schema_name }}'
    AND table_type = 'VIEW'
    AND {{ syntax.like('table_name', prefix + "%") }}
