-- noqa: disable=all
-- This sql was autogenerated as a reference example using the library
-- CLI. Its format is tied to the specific database it was run against,
-- and it may not be correct for all databases. Use the CLI's build 
-- option to derive the best SQL for your dataset.

-- ###########################################################

CREATE TABLE example_nlp__range_labels AS
SELECT
    'x' AS note_ref,
    'x' AS subject_ref,
    'x' AS label,
    'x' AS span,
    'x' AS origin
WHERE 1=0 -- empty table
